#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "trie.h"

void print_val(char * key, void * data)
   {
   printf("%s\t\t%s\n", key, data);
   }

int main(int argc, char ** argv)
   {
   struct trie * trie = trie_new();

   char * words[] = { "hello", "interjection (used to express a greeting, answer a telephone, or attract attention.)",
		      "help", "to give or provide what is necessary to accomplish a task or satisfy a need.",
		      "helper", "a person or thing that helps  or gives assistance, support, etc",
		      "helping", "the act of a person or thing that helps.",
		      "an", "indefinite article. the form of a before an initial vowel sound.",
		      "a", "indefinite article.  Used before an initial consonant sound.",
		      "anne", "a female name",
		      "any", "one, a, an, or some; one or more without specification or identification",
		      "anyway", "in any case; anyhow; nonetheless; regardless",	
		      "anna", "a female name",
		      "anyhow", "in any  way whatever",
		      "anybody", "any person",
		      "Apple", "A computer manufacturer.",
		      "ABLE", "having necessary power, skill, resources, or qualifications",
		      NULL, NULL
   };
   int i = 0;
   while (words[i])
      {
      trie_insert(trie, words[i], words[i+1]);
      i += 2;
      }

   trie_dump_raw(trie);
   trie_dump(trie);
   trie_walk_keys(trie, print_val);
   printf("trie is storing %zd words in %zd keys\n", trie_count(trie), trie_size(trie));

   ////////////////////////////////////////////
   printf("===================================================\n");
   char word[100];;
   strcpy(word, "any");
   printf("searching for %s\n", word);

   void * data = trie_find(trie, word);
   if (NULL == data)
      printf("Failed to find 'any'\n");
   else
      printf("any: %s\n", (char *) data);


   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "an");
   printf("Looking for completions of '%s':\n", word);
   struct list * list = trie_get_subkeys(trie, word, 0);
   if (NULL == list)
      printf("Nothing found for '%s'\n", word);
   else
      {
      printf("list size is %d\n", list_size(list));
      struct list_iterator * it = list_begin(list);
      struct list_iterator * next = it;
      while (NULL != next)
	 {
	 printf("%s\n", list_value(it));
	 next = list_next(it);
	 }
      trie_free_subkey_list(list);
      }

   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "a");
   printf("Looking for first 5 completions of '%s':\n", word);
   list = trie_get_subkeys(trie, word, 5);
   if (NULL == list)
      printf("Nothing found for '%s'\n", word);
   else
      {
      printf("list size is %d\n", list_size(list));
      struct list_iterator * it = list_begin(list);
      struct list_iterator * next = it;
      while (NULL != next)
	 {
	 printf("%s\n", list_value(it));
	 next = list_next(it);
	 }
      trie_free_subkey_list(list);
      }

   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "anybody");
   printf("remove '%s'\n", word);
   trie_remove(trie, word);

   strcpy(word, "ann");
   printf("remove '%s'\n", word);
   trie_remove(trie, word);

   strcpy(word, "help");
   printf("remove '%s'\n", word);
   trie_remove(trie, word);

   strcpy(word, "a");
   printf("remove '%s'\n", word);
   trie_remove(trie, word);

   trie_dump(trie);
   printf("trie is storing %zd words in %zd keys\n", trie_count(trie), trie_size(trie));

   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "anybody");
   printf("re-inserting '%s'\n", word);
   trie_insert(trie, word, "fake definition");
   trie_dump(trie);
   printf("trie is storing %zd words in %zd keys\n", trie_count(trie), trie_size(trie));

   exit(EXIT_SUCCESS);
   }
